<?php

namespace TDevAgency\Cms\Controllers\Admin;

use Illuminate\Contracts\View\Factory;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\View;
use Illuminate\Validation\ValidationException;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Cms\Events\PostSaveEvent;
use TDevAgency\Cms\Models\BaseModels\Post;
use TDevAgency\Cms\Models\BaseModels\PostConnection;
use TDevAgency\Cms\Models\BaseModels\PostLocalization;
use TDevAgency\Cms\Traits\CategoryTrait;
use TDevAgency\Cms\Traits\PostHelper;
use Throwable;

class PostsController extends Controller
{
    use CategoryTrait, PostHelper;

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function index(Request $request)
    {
        return $this->displayList(Post::POST_TYPE, $request);
    }

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $categories = Post::where('post_type', Post::CATEGORY_TYPE)->with(['localizations'])->get();
        $post = new Post();
        $post->post_type = Post::POST_TYPE;
        return View::make('cms::admin.screens.posts.single')
            ->with('categories', $categories)
            ->with('selectedCategories', [])
            ->with('post', $post);
    }

    /**
     * @param Request $request
     * @return RedirectResponse
     * @throws ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validatorsArray());


        $post = new Post();
        try {
            DB::transaction(function () use ($request, &$post) {
                $this->postPrepare($request, $post);
                $post->post_type = Post::POST_TYPE;
                $post->save();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $post);
                }

                if ($request->has('category')) {
                    foreach ($request->get('category') as $id => $category) {
                        $postCategory = new PostConnection();
                        $postCategory->post_parent_id = $id;
                        $postCategory->post_id = $post->id;
                        $postCategory->save();
                    }
                }
                Event::dispatch(new PostSaveEvent($post, $request));
            });
            return redirect()->route('cms.posts.index')->with('toast', [
                'title' => trans('cms::admin.toast.created'),
                'status' => 'success'
            ]);
        } catch (Throwable $e) {
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_created'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param $id
     */
    public function show($id)
    {
        //
    }

    /**
     * @param Post $post
     * @return Factory|\Illuminate\View\View
     */
    public function edit($post)
    {
        $post = $this->checkValue($post);
        [$categories, $selectedCategories] = $this->categories($post->id);
        $post->setRelation('localizations', $post->localizations->mapWithKeys(function (PostLocalization $postLocalization) {
            return [$postLocalization->lang => $postLocalization];
        }));
        return view('cms::admin.screens.posts.single')
            ->with('post', $post)
            ->with('categories', $categories)
            ->with('selectedCategories', $selectedCategories);
    }

    /**
     * @param $post
     * @return Post
     */
    private function checkValue($post)
    {
        if (!is_object($post)) {
            return Post::where('id', $post)->firstOrFail();
        }
        return $post;

    }

    /**
     * @param Request $request
     * @param Post $post
     * @return RedirectResponse
     * @throws ValidationException
     */
    public function update(Request $request, $post)
    {
        $post = $this->checkValue($post);
        $this->validate($request, $this->validatorsArray($post->id));

        try {
            DB::transaction(function () use ($request, &$post) {
                $this->postPrepare($request, $post);
                $post->update();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $post);
                }
                if ($request->has('category')) {
                    PostConnection::whereNotIn('post_parent_id', $request->get('category'))->delete();
                    foreach ($request->get('category') as $key => $value) {
                        PostConnection::updateOrCreate(
                            [
                                'post_id' => $post->id,
                                'post_parent_id' => $key,
                            ],
                            [
                                'post_id' => $post->id,
                                'post_parent_id' => $key,
                            ]
                        );
                    }
                }

                Event::dispatch(new PostSaveEvent($post, $request));
            });
            return redirect()->route('cms.posts.index')->with('toast', [
                'title' => trans('cms::admin.toast.updated'),
                'status' => 'success'
            ]);
        } catch (Throwable $e) {
            return redirect()->back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_updated'),
                'status' => 'danger'
            ]);
        }

    }

    /**
     * @param Post $post
     * @return RedirectResponse
     * @throws Throwable
     */
    public function destroy($post)
    {
        $post = $this->checkValue($post);

        DB::transaction(function () use ($post) {
            PostLocalization::where('post_id', $post->id)->delete();
            PostConnection::where('post_id', $post->id)->delete();
            Post::where('id', $post->id)->delete();
        });
        return back()->with('toast', [
            'title' => trans('cms::admin.toast.deleted'),
            'status' => 'success'
        ]);
    }
}
